/**************************************************************************\
*          Copyright (c) 1995 INRIA Sophia Antipolis, FRANCE.              *
*                                                                          *
* Permission to use, copy, modify, and distribute this material for any    *
* purpose and without fee is hereby granted, provided that the above       *
* copyright notice and this permission notice appear in all copies.        *
* WE MAKE NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS     *
* MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS   *
* OR IMPLIED WARRANTIES.                                                   *
\**************************************************************************/
/**************************************************************************\
* 	                						   *
*  File              : audio.c       	                		   *
*  Author            : Hugues CREPIN					   *
*  Last Modification : 1995/2/15                                           *
*--------------------------------------------------------------------------*
*  Description       : The procedure what receive and play the sound.      *
*                      Version 1.0                                         *
*--------------------------------------------------------------------------*
*        Name	        |    Date   |          Modification                *
*--------------------------------------------------------------------------*
*                                                                          *
\**************************************************************************/

#include <stdio.h>

#ifdef __FreeBSD__
#include <stdlib.h>
#else
#include <malloc.h>
#endif
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <signal.h>
#include <fcntl.h>

#include "general_types.h"
#include "protocol.h"
#include "audio.h"
#include "rtp.h"

#ifdef CRYPTO
#include "ivs_secu.h"
#endif /* CRYPTO */

#define frame_length 320
#define P_MAX 2000

#define PCM_64                  0
#define ADPCM_32                1
#define VADPCM                  2

#ifndef NO_AUDIO

static int nb_stations=0;
static int LONGUEUR_DEMANDE=2000; 

static BOOLEAN DEBUT = TRUE;
static BOOLEAN AUDIO_SET;
static BOOLEAN AUDIO_DROPPED;
static BOOLEAN ORDO_REQUESTED = FALSE;

static BOOLEAN MICRO_MUTED=TRUE;
static BOOLEAN Audio_Quit=FALSE;


static int encoding,squelch;
static u_char ttl;

static int alen_addr_r;
static struct sockaddr_in addr_r;
static int asock_r;

static BOOLEAN UNICAST=TRUE;
static BOOLEAN DEFAULT_DECODE=TRUE;

static char IP_GROUP[256], AUDIO_PORT[5];

static int VOLUME=0;
static BOOLEAN SPEAKER;

extern BOOLEAN DEBUG_MODE;
extern S_PARAM param;

#define NB_MAX_ST 10000

enum mode_t {NORMAL,DETECTION_PIC};

typedef struct
{
  double debut,var,tps_init,ni,ni1,ni2,di,dvi,pi,ni_start;
  enum mode_t mode;
  u_int32 nb_dvi;
} Param_Algo_t;

typedef struct paquet_p
{
  u_int8   validaudio:1;          /* set if audio if still valid   */
  u_int8   beginoftalkspurt:1;    /* set if the next packet is a   */
                                  /* begin of a talkspurt          */
  u_int8   reserved:6;
  u_int8   reserved2;             /* for 32 bits frontiere         */
  u_int16  lastarrived;           /* The last paquet arrived       */
  u_int32  actualplay;            /* the actual playing position   */
  double   sendtime;              /* The time when the packet was sent */
  double   arrivaltime;           /* The time when the packet was arrived */
  u_int32  playout;               /* The playout in byte*/
  u_int32  longueur;              /* Length of frame*/
  u_char   *data;                 /* data of the frame */
  struct   paquet_p *suivant;     /* the next paquet */
} paquet_t;

typedef struct
{
  u_long  sin_addr;        /* packet has been received from this address  */
  u_short identifier;         /* source identifier, set to 0 if no bridge */
  u_int8   audio_encoding:1;           /* set if the station encodes audio */
  u_int8   audio_decoding:1;             /* set if audio decoding selected */
  u_int8   audio_ignored:1;                     /* set if audio is ignored */
  u_int8   alwaysinit:1;             /* set if the station was always init */
  u_int8   reserved:4;
  u_int8   reserved2;
  Param_Algo_t parameter;
  paquet_t * p_paquet;
} STATION_AUDIO;

static STATION_AUDIO table[NB_MAX_ST];

void recoit();
static void Timeout();
static void joue();
static int ordonnance();
static short pcmu_to_l14();
static u_char l14_to_pcmu();
static void mix_init();
static void mix2_pcmu();
static paquet_t * Insere();
static paquet_t * Remove();
static paquet_t * RemoveEnTete();
static paquet_t * InitList();
static paquet_t * PurgeList();
 
/* Returns the decoded frame length */
static unsigned int DecodeAudio(inbuf, soundel, format, outbuf)
     unsigned char *inbuf, *outbuf;
     int soundel;
     u_char format;
{
    int len;

    if (soundel<=0)
      return 0;

    switch(format){
    case RTP_PCM_CONTENT:
      memcpy((char *)outbuf,(char *)inbuf,soundel);
      len=soundel;
      break;
    case RTP_ADPCM_CONTENT:
      len = soundel*2;
      adpcm_decoder(inbuf, outbuf, len);
      break;
    case RTP_VADPCM_CONTENT:
      len=vadpcm_decoder(inbuf, outbuf, soundel);
      break;
    default:
      return 0;
    }
    return len;
  }



void SetMicroMuted()
{
  MICRO_MUTED=TRUE;
}


void UnSetMicroMuted()
{
  MICRO_MUTED=FALSE;
}

BOOLEAN IsMicroMuted()
{
  return MICRO_MUTED;
}

void SetEncoding(encod)
     int encod;
{
  encoding=encod;
}

int Encoding()
{
  return encoding;
}

void SetSquelch(squ)
     int squ;
{
  squelch=squ;
}

int Squelch()
{
  return squelch;
}

void AudioQuit()
{
  Audio_Quit=TRUE;
}

static void ToggleMicroOn()
{
  soundplayvol(VOLUME);  
}

static void ToggleMicroOff()
{
  soundplayvol(0);  
}

void SetVolume(vol)
     int vol;
{
  VOLUME=vol;
}


int AudioEncode(coding, MICRO_MUTE)
     int coding;
     BOOLEAN MICRO_MUTE;
{
  static int sock, len_addr;
  static struct sockaddr_in addr;
  unsigned int inbuf[P_MAX/4], outbuf[P_MAX/4];
  static unsigned char grabber[P_MAX*2];
  static unsigned char *ptr_grabber=grabber;
  static int counter=0; /* the number of sample */
  static unsigned int *buf;
  rtp_hdr_t *h; /* fixed RTP header */
  static u_int16 cpt=1;
  static u_short port_audio_source;
  static u_int8 end_of_talkspurt = 0 ; /* mark of end of talkspurt */
  static u_short end_emmit = 2;
  static BOOLEAN FIRST=TRUE, SOUND_RECORD=FALSE;
  int format=0;
  struct timeval send_time;
  static BOOLEAN Precedent_state=TRUE; /* precedent_state of the micro mute */
  int len_packet;
  char *buf_to_send;
#ifdef CRYPTO
  u_int buf_crypt[P_MAX/4 + 2];
#endif


  h = (rtp_hdr_t *)outbuf;
  h->ver = 1;
  h->channel = 0;
  h->p = 0;
  h->s = 0; /* now */
  h->format = (u_int8)(0x0003F & encoding);
  h->seq = htons(cpt);

  if (FIRST) {
      FIRST=FALSE;
      Audio_Quit=FALSE;
      switch (coding) {
      case PCM_64:
	format = RTP_PCM_CONTENT;
	break;
      case ADPCM_32:
	format = RTP_ADPCM_CONTENT;
	break;
      case VADPCM:
	format = RTP_VADPCM_CONTENT;
	break;
      }
      SetEncoding(format);
      SetSquelch(squelch);
      Audio_Quit=FALSE;
      sock = CreateSendSocket(&addr, &len_addr, AUDIO_PORT, IP_GROUP, &ttl,
			      &port_audio_source, UNICAST, param.ni);

#if defined(SPARCSTATION) || defined(SGISTATION) || defined(I386AUDIO)
      if(issoundrecordinit() == FALSE){
	if(!soundrecordinit()){
	  fprintf(stderr, "audio_encode: Cannot open audio device.\n");
	  close(sock);
	  FIRST=TRUE;
	  return 0;
	}
      }
#ifdef SPARCSTATION
      soundplayflush();
      soundrecflush();
#endif
#endif
#ifdef AF_AUDIO
      if(!audioInit()){   
	fprintf(stderr,"audio_coder: cannot connect to AF AudioServer\n");
      }
#endif /* AF_AUDIO */
      
      if (MICRO_MUTE)
	SetMicroMuted();
      else
	UnSetMicroMuted();

      if (soundrecord()) {
	  SOUND_RECORD=TRUE;
	  if(encoding == RTP_PCM_CONTENT){
	    buf = &outbuf[2];
	  }else{
	    buf = inbuf;
	  }
	}
      ptr_grabber=grabber;
      counter=0;
#if defined(SPARCSTATION) || defined(SGISTATION) || defined(I386AUDIO)
      soundrecordterm();
#endif
    }/* end of if(FIRST) */
  if (SOUND_RECORD) {
      int soundel;

      if (Audio_Quit) {
	  Audio_Quit=FALSE;
	  SOUND_RECORD=FALSE;
	  FIRST=TRUE;
	  close(sock);
	  SetMicroMuted();
	  soundstoprecord();
#if defined(SPARCSTATION) || defined(SGISTATION) || defined(I386AUDIO)	  
	  soundrecordterm();
#endif	
  	  ToggleMicroOn();
	  Precedent_state=TRUE;
#ifdef SPARCSTATION
	  soundrecflush();
#endif
	  return 0;
	}

      h->format= (u_int8)(0x0003F & encoding);
      if(encoding == RTP_PCM_CONTENT) {
	  buf = &outbuf[2];
      } else {
	  buf = inbuf;
      }
      
      if(MICRO_MUTED){
	  if (end_emmit < 2) {
	    /* Flush if something remains in the queue */
	    {
	      register int i;
	      while(counter>=frame_length){
		soundel=frame_length;
		memcpy((char *)buf, (char *)grabber, frame_length);
		switch(encoding){
		case RTP_PCM_CONTENT:
		  break;
		case RTP_ADPCM_CONTENT:
		  adpcm_coder(inbuf, &outbuf[2], soundel);
		  soundel /= 2;
		  break;
		case RTP_VADPCM_CONTENT:
		  soundel = vadpcm_coder(inbuf, &outbuf[2], soundel);
		  break;
		}
		end_emmit=0;
		h->seq = htons(cpt);
		h->s=end_of_talkspurt;
		end_of_talkspurt=0;
		gettimeofday(&send_time, (struct timezone *)NULL);
		h->ts_sec = htons(send_time.tv_sec & 0xffff);
		h->ts_usec = htons((send_time.tv_usec >> 4) & 0xffff);

		len_packet = soundel + 8;
		buf_to_send = (char *)outbuf;
#ifdef CRYPTO
		if(ENCRYPTION_ENABLED()){
#ifdef DEBUG_CRYPTO
		    test_key("audio_coder");
#endif
		    len_packet = method[current_method].crypt_rtp(
			(char *)outbuf, len_packet, s_key, len_key, 
			(char *)buf_crypt);
		    buf_to_send = (char *)buf_crypt;
		}	    
#endif /* CRYPTO */	    
		if(sendto(sock, buf_to_send, len_packet, 0,
			  (struct sockaddr *) &addr, len_addr) != len_packet)
		  {
		    perror("audio_coder: when sending datagram packet");
		  }
		cpt = (u_int16)((cpt +1) % 0xFFFF);
		if (counter>frame_length)
		  {
		    for (i=0;i<counter-frame_length;i++)
		      grabber[i]=grabber[i+frame_length];
		  }
		counter-=frame_length;
	      }
	    }
	    
	    h->seq = htons(cpt);
	    h->s=1;
	    end_of_talkspurt=0;
	    gettimeofday(&send_time, (struct timezone *)NULL);
	    h->ts_sec =  htons(send_time.tv_sec & 0xffff);
	    h->ts_usec = htons((send_time.tv_usec >> 4) & 0xffff);
	    len_packet = 8;
	    buf_to_send = (char *)outbuf;
#ifdef CRYPTO
	    if(ENCRYPTION_ENABLED()){
#ifdef DEBUG_CRYPTO
	      test_key("audio_coder");
#endif
	      len_packet = method[current_method].crypt_rtp(
		  (char *)outbuf, len_packet, s_key, len_key, 
		  (char *)buf_crypt);
	      buf_to_send = (char *)buf_crypt;
	    }	    
#endif /* CRYPTO */	    
	    if(sendto(sock, (char *)buf_to_send, len_packet, 0,
		      (struct sockaddr *) &addr, len_addr) != len_packet){
		perror("audio_coder: when sending datagram packet");
	    }
	    cpt = (u_int16)((cpt + 1) % 0xFFFF);
	    end_emmit=2;
	    ptr_grabber=grabber;
	    counter=0;
#if defined(SPARCSTATION) || defined(SGISTATION) || defined(I386AUDIO)
	    soundrecordterm();
#endif
	    ToggleMicroOn();
#ifdef SPARCSTATION
	    soundplayflush();
	    soundrecflush();
#endif
	  }
	if (!Precedent_state)
	  ToggleMicroOn();
	Precedent_state=TRUE;
	return 1;
      }/* if micromuted */
      if (Precedent_state) {
	  if (SPEAKER)
	    ToggleMicroOff();
#ifdef SPARCSTATION
	  soundplayflush();
	  soundrecflush();
#endif
	  Precedent_state=FALSE;
	}
      /* Get the sound ... */
      {
#if defined(SPARCSTATION) || defined(SGISTATION) || defined(I386AUDIO)
	  if(issoundrecordinit() == FALSE) {
	      if(!soundrecordinit()) {
		fprintf(stderr,
			"audio_encode: 2nd call Cannot open audio device.\n");
	      }
	  }
#endif
	ptr_grabber=grabber+counter;
	counter += soundgrab(ptr_grabber, 2*P_MAX-counter);
      }
      if (counter < frame_length){
	return 1;
      }
      soundel=frame_length;
      memcpy((char *)buf, (char *)grabber, frame_length);
      
      if (soundel > 0) {
	register u_char *start = (u_char *)buf;
	register int i;
	int squelched = (squelch > 0);
	
	
	/* If entire buffer is less than squelch, ditch it. */
	if (squelch > 0) {
	  for (i=0; i<soundel; i++) {
	    if ((int)((*start++ & 0x7F) ^ 0x7F) > squelch) {
	      squelched = 0;
	      break;
	    }
	  }
	}
	if (!squelched){
	  switch(encoding){
	  case RTP_PCM_CONTENT:
	    break;
	  case RTP_ADPCM_CONTENT:
	    adpcm_coder(inbuf, &outbuf[2], soundel);
	    soundel /= 2;
	    break;
	  case RTP_VADPCM_CONTENT:
	    soundel = vadpcm_coder(inbuf, &outbuf[2], soundel);
	    break;
	  }
	  end_emmit=0;
	  h->seq = htons(cpt);
	  h->s=end_of_talkspurt;
	  end_of_talkspurt=0;
	  gettimeofday(&send_time, (struct timezone *)NULL);
	  h->ts_sec = htons(send_time.tv_sec & 0xffff);
	  h->ts_usec = htons((send_time.tv_usec >> 4) & 0xffff);

	  len_packet = soundel+8;
	  buf_to_send = (char *)outbuf;
#ifdef CRYPTO
	  if(ENCRYPTION_ENABLED()){
#ifdef DEBUG_CRYPTO
	    test_key("audio_coder");
#endif
	    len_packet = method[current_method].crypt_rtp(
		(char *)outbuf, len_packet, s_key, len_key, 
		(char *)buf_crypt);
	    buf_to_send = (char *)buf_crypt;
	  }	    
#endif /* CRYPTO */
	  if(sendto(sock, (char *)buf_to_send, len_packet, 0,
		    (struct sockaddr *) &addr, len_addr) != len_packet){
	      perror("audio_coder: when sending datagram packet");
	  }
	  
	  cpt = (u_int16)((cpt + 1) % 0xFFFF);
	}
	else {
	    if (end_emmit < 2) {
		end_emmit++;
		if (end_emmit==2) {
		    end_of_talkspurt = 1;
		    Precedent_state=TRUE;
		}
		switch(encoding){
		case RTP_PCM_CONTENT:
		  break;
		case RTP_ADPCM_CONTENT:
		  adpcm_coder(inbuf, &outbuf[2], soundel);
		  soundel /= 2;
		  break;
		case RTP_VADPCM_CONTENT:
		  soundel = vadpcm_coder(inbuf, &outbuf[2], soundel);
		  break;
		}
		h->seq = htons(cpt);
		h->s=end_of_talkspurt;
		end_of_talkspurt=0;
		gettimeofday(&send_time, (struct timezone *)NULL);
		h->ts_sec = htons(send_time.tv_sec & 0xffff);
		h->ts_usec =  htons((send_time.tv_usec >> 4) & 0xffff);
		len_packet = soundel+8;
		buf_to_send = (char *)outbuf;
#ifdef CRYPTO
		if(ENCRYPTION_ENABLED()){
#ifdef DEBUG_CRYPTO
		    test_key("audio_coder");
#endif
		    len_packet = method[current_method].crypt_rtp(
			(char *)outbuf, len_packet, s_key, len_key, 
			(char *)buf_crypt);
		    buf_to_send = (char *)buf_crypt;
		}	    
#endif /* CRYPTO */
		if(sendto(sock, (char *)buf_to_send, len_packet, 0,
			  (struct sockaddr *) &addr, len_addr) != len_packet){
		    perror("audio_coder: when sending datagram packet");
		}
		cpt = (u_int16)((cpt +1) % 0xFFFF);
	      }
	  }
	if (counter>frame_length)
	  {
	    for (i=0;i<counter-frame_length;i++)
	      grabber[i]=grabber[i+frame_length];
	  }
	counter -= frame_length;
      }
    }
return 1;
}



/* --------------------Functions for the receiver-------------------------*/

void restoreinitialstate()
{
  register int i;

  for (i=0;i<nb_stations;i++)
    table[i].audio_decoding=!table[i].audio_ignored;
  
}

static int SinToIndice(address,ident)
     u_long  address;        /* packet has been received from this address  */
     u_short ident;             /* source identifier, set to 0 if no bridge */
{
  register int i;

  for (i=0;i<nb_stations;i++)
    if (table[i].sin_addr==address && table[i].identifier==ident)
      return i;
  return -1;
}

static void IndiceToSin(indice,address,ident)
     int indice;                                  /* the indice in the table */
     u_long  * address;       /* packet has been received from this address  */
     u_short * ident;            /* source identifier, set to 0 if no bridge */
{
    if (indice<=nb_stations) {
      *address=table[indice].sin_addr;
      *ident=table[indice].identifier;
    }
    else {
      *address=0;
      *ident=0;
    }
}
     
void SetAudioIgnored(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
{
  int indice;

  indice=SinToIndice(address,ident);
  if (indice>=0)
    table[indice].audio_ignored=TRUE;
}

void UnSetAudioIgnored(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
{
  int indice;

  indice=SinToIndice(address,ident);
  if (indice>=0)
    table[indice].audio_ignored=FALSE;
}


BOOLEAN IsAudioIgnored(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
{
  int indice;

  indice=SinToIndice(address,ident);
  if (indice>=0)
    return table[indice].audio_ignored;
  else
    return DEFAULT_DECODE;
}

void SetAudioDecoding(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
{
  int indice;

  indice=SinToIndice(address,ident);
  if (indice>=0)
    table[indice].audio_decoding=TRUE;
}

void UnSetAudioDecoding(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;     /* source identifier, set to 0 if no bridge */
			
{
  int indice;

  indice=SinToIndice(address,ident);
  if (indice>=0)
    table[indice].audio_decoding=FALSE;
}  

BOOLEAN IsAudioDecoding(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;     /* source identifier, set to 0 if no bridge */
			
{
  int indice;

  indice=SinToIndice(address,ident);
  if (indice>=0)
    return table[indice].audio_decoding;
  return FALSE;
}  

void SetAudioEncoding(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
{
  int indice;

  indice=SinToIndice(address,ident);
  if (indice>=0)
    table[indice].audio_encoding=TRUE;
}

void UnSetAudioEncoding(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
{
  int indice;

  indice=SinToIndice(address,ident);
  if (indice>=0)
    table[indice].audio_encoding=FALSE;
}

BOOLEAN IsAudioEncoding(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
{
  int indice;

  indice=SinToIndice(address,ident);
  if (indice>=0)
    return table[indice].audio_encoding;
  return FALSE;
}

BOOLEAN IsStationDeclare(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
{
  int indice;

  indice=SinToIndice(address,ident);
  if (indice>=0)
    return TRUE;
  else
    return FALSE;
}


void Listen(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
{
  int indice;

  indice=SinToIndice(address,ident);
  if (indice>=0)
    {
      table[indice].audio_ignored=FALSE;
      table[indice].audio_decoding=TRUE;
    }
}


void Mute(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
{
  int indice;

  indice=SinToIndice(address,ident);
  if (indice>=0)
    {
      table[indice].audio_ignored=TRUE;
      table[indice].audio_decoding=FALSE;
    }
}


BOOLEAN NewAudioStation(address,ident,audio_encod,audio_decod,audio_igno)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
     u_int8   audio_encod;               /* set if the station encodes audio */
     u_int8   audio_decod;                 /* set if audio decoding selected */
     u_int8   audio_igno;                         /* set if audio is ignored */
     
{
  int indice;
  paquet_t paquet;

  indice=SinToIndice(address,ident);
  if (indice>=0) {
      table[indice].audio_encoding=audio_encod;
      table[indice].audio_decoding=audio_decod;
      table[indice].audio_ignored=audio_igno;
      return TRUE;
    }  else
      if (nb_stations<NB_MAX_ST) {
	table[nb_stations].sin_addr=address;
	table[nb_stations].identifier=ident;
	table[nb_stations].audio_encoding=audio_encod;
	table[nb_stations].audio_decoding=audio_decod;
	table[nb_stations].audio_ignored=audio_igno;
	table[nb_stations].alwaysinit=TRUE;
	paquet.beginoftalkspurt = 1;
	paquet.validaudio = 0;
	paquet.actualplay = 0; 
	paquet.sendtime = 0;
	paquet.lastarrived = 0;
	paquet.arrivaltime = 0;
	paquet.playout = 0; 
	paquet.longueur = 0;
	paquet.data = 0;
	table[nb_stations].p_paquet = (paquet_t *)InitList();
	table[nb_stations].p_paquet = 
	    (paquet_t *)Insere(table[nb_stations].p_paquet,&paquet);
	table[nb_stations].parameter.debut=0;
	table[nb_stations].parameter.var=0;
	table[nb_stations].parameter.tps_init=0;
	table[nb_stations].parameter.ni=0;
	table[nb_stations].parameter.ni1=0;
	table[nb_stations].parameter.ni2=0;
	table[nb_stations].parameter.di=0;
	table[nb_stations].parameter.dvi=0;
	table[nb_stations].parameter.pi=0;
	table[nb_stations].parameter.ni_start=0;
	table[nb_stations].parameter.nb_dvi=1;
	nb_stations++;
	return TRUE;
      }
    return FALSE;
}

BOOLEAN RemoveAudioStation(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
{
  int indice;

  indice=SinToIndice(address,ident);
  if (indice>=0) {
      PurgeList(table[indice].p_paquet);
      for (;indice<nb_stations-1;indice++)
	table[indice]=table[indice+1];
      nb_stations--;
      return TRUE;
    }
  return FALSE;
}


void RemoveAudioAll()
{
  register int i;

  for (i=0;i<nb_stations;i++)
      PurgeList(table[i].p_paquet);
  bzero((char *)table,sizeof(table));
  nb_stations=0;
}


void InitAudio(default_decoding)
     BOOLEAN default_decoding;
{
 bzero((char *)table,sizeof(table));
 DEFAULT_DECODE=default_decoding;
}




void InitAudioSocket(port,group,Ttl,Squelch,Unicast)
     char *port;
     char *group; /* IP address style "225.23.23.6" = "a4.a3.a2.a1" */
     u_char Ttl;
     int Squelch;
     BOOLEAN Unicast;
{
  strcpy(AUDIO_PORT,port);
  strcpy(IP_GROUP,group);
  UNICAST=Unicast;
  ttl=Ttl;
  squelch=Squelch;
}


void AudioSocketReceive(audio_socket)
     int *audio_socket;
{
  *audio_socket=asock_r = CreateReceiveSocket(&addr_r, &alen_addr_r,
					      AUDIO_PORT, IP_GROUP, UNICAST, 
					      param.ni);
}



static BOOLEAN ShallIDecode(sin_addr, identifier, i_station)
     u_long sin_addr;
     u_short identifier;
     int *i_station;
{
  int indice;
 
  indice=SinToIndice(sin_addr,identifier);
  *i_station=indice;
  if (indice>=0) {
      if(table[indice].audio_decoding)
	return(TRUE);
      else
	return(FALSE);
  }
  return(FALSE);  
}


BOOLEAN AnyAudioDecoding()
{
  register int i;

  for(i=0; i<nb_stations; i++){
    if(table[i].audio_decoding && !table[i].audio_ignored)
      return(TRUE);
  }
  return(FALSE);
}


void SetSpeaker(speak)
     BOOLEAN speak;
{
  SPEAKER=speak;
}



/*-------------------------ManageAudio--------------------------------------*/


int ManageAudio(audio_set, audio_dropped)
     BOOLEAN audio_set;
     BOOLEAN audio_dropped;
  {
  int NOSTATION;
  static u_char BUFFER_AUDIO[BUFFER_AUDIO_LENGTH];
  static int OLDLOCALPLAY=0, GLOBALPLAY=0;

  AUDIO_SET=audio_set;
  AUDIO_DROPPED=audio_dropped;
  if(ORDO_REQUESTED || !DEBUT){
    ORDO_REQUESTED = FALSE;
    NOSTATION=ordonnance(BUFFER_AUDIO,&OLDLOCALPLAY,&GLOBALPLAY);
    Timeout(NOSTATION,OLDLOCALPLAY,GLOBALPLAY);
    joue(BUFFER_AUDIO,&OLDLOCALPLAY,&GLOBALPLAY);
  }
  return DEBUT;
}



/* This procedure returns the time at which the audio must be play */
static double algo(parametre,recepteur,emetteur,bot)
     Param_Algo_t * parametre;
     double recepteur;
     double emetteur;
     int bot;

{
/* The first time we set the parameter at the adapted value */
    if (parametre->debut==0.0) {
      parametre->ni=recepteur-emetteur;
      parametre->ni1=parametre->ni;
      parametre->di=parametre->ni;
      parametre->dvi=0.0;
      parametre->pi=recepteur;
      parametre->debut=emetteur;
      parametre->tps_init=parametre->pi;
      parametre->ni_start=parametre->ni;
      parametre->nb_dvi=1;
      parametre->mode=NORMAL;
      parametre->ni2=0.0;
    } else {
/* After we run the algorithm */      
      parametre->ni=recepteur-emetteur;
/* We detect if it is a pike */      
      if (parametre->mode==NORMAL) {
	  if (parametre->ni>parametre->ni1)
	    parametre->mode=DETECTION_PIC;
      } else {
	  if (parametre->ni1>parametre->ni){
	      parametre->mode=NORMAL;
	      (parametre->nb_dvi)++;
	      if (parametre->nb_dvi==0)
		parametre->dvi=0;
	      else
		parametre->dvi=parametre->dvi*
		  (1-1/(float)(parametre->nb_dvi))+
		  ((parametre->ni1-parametre->ni
		    +parametre->ni1-parametre->ni2)/2.0)/
		    (float)(parametre->nb_dvi);
	    }
	  else {
	      parametre->ni_start=parametre->ni;
	  }
      }
      parametre->ni2=parametre->ni1;
      parametre->ni1=parametre->ni;

      if (bot==1) {
	  parametre->debut=emetteur;
	  parametre->pi=parametre->dvi+parametre->ni_start+emetteur;
	  parametre->tps_init=parametre->pi;
      } else
	  parametre->pi=parametre->tps_init+emetteur-parametre->debut;
    }
  return parametre->pi;
}



/*Reception*/
/* This procedure take data and insert them at the good place */
/* for each station */
/* asock_r is the audio socket recieve number */

void reception(sin_addr,identifi)
     u_long * sin_addr;
     u_short * identifi;
{
  static struct sockaddr_in from;
  static int fromlen;
  u_int pbuf[P_MAX/4],poutbuf[BUFFER_AUDIO_LENGTH/4];
  u_char * outbuf = (u_char *)poutbuf;
  rtp_hdr_t *h; /* fixed RTP header */
  rtp_t *option; /* RTP option (CRSC) */
  u_long * pt;
  u_short identifier;
  int lr, i_station=-1;
  u_short seq;
  BOOLEAN DECODING_REQUIRED;
  BOOLEAN ERROR=FALSE;
  static struct timeval realtime, sendtime;
  u_int16 longueur;
  static paquet_t paquet;
  static BOOLEAN FIRST = TRUE; 
  char name[LEN_MAX_NAME];
  struct hostent *host;
#ifdef DEBUG
  struct trace {
    unsigned int numero;
    int begin_of_t;
    int longueur;
    double emetteur;
    double recepteur;
    u_long adress;
  } TRACE;
  static FILE *file_trace;
#endif

  *sin_addr=*identifi=0;
  
  if (FIRST) {
      fromlen = sizeof(from);
      FIRST = FALSE;
      mix_init();
#ifdef DEBUG      
      if ((file_trace=fopen("mesure.dat","w"))==0) {
	  fprintf(stderr,"Impossible d'ouvrir le fichier de mesure \n");
	  exit(8);
      }
#endif      

  }

  bzero((char *)&from, fromlen);
  if((lr=recvfrom(asock_r, (char *)pbuf, P_MAX, 0, 
		  (struct sockaddr *)&from, &fromlen)) < 0) {
    if(DEBUG_MODE){
      perror("recvfrom'audio_decode:");
    }
    return;
  }
  
#ifdef CRYPTO
  if(ENCRYPTION_ENABLED()){
#ifdef DEBUG_CRYPTO
      test_key("audio_decoder");
#endif
      lr = method[current_method].uncrypt_rtp((char *)pbuf, lr, s_key, 
					      len_key, (char *)pbuf);
  }
#endif /* CRYPTO */

  h = (rtp_hdr_t *)pbuf;
  pt = (u_long *) (h+1);
  identifier=0;
  if(h->ver != RTP_VERSION){
    if(DEBUG_MODE){
      fprintf(stderr, "ParseRTCP: Bad RTP version %d : ", h->ver);
    }
    ERROR = TRUE;
    goto after_parsing;
  }
  if(h->channel){
    if(DEBUG_MODE){
      fprintf(stderr, "ParseRTCP: Bad RTP channel %d : ", h->channel);
    }
    ERROR = TRUE;
    goto after_parsing;
  }
  switch(h->format) {
  case RTP_PCM_CONTENT:
  case RTP_ADPCM_CONTENT:
  case RTP_VADPCM_CONTENT:
    break;
  default:
    if(DEBUG_MODE){
      fprintf(stderr, "Parse audio: Bad RTP payload %d : ", h->format);
    }
    ERROR = TRUE;
    goto after_parsing;
  }

  if(h->p) {
      /* An option field is added (SSRC) */
      do {
	  option = (rtp_t *)pt;
	  pt += option->generic.length;
	  if ((char *)pt -(char *)h>lr || option->generic.length == 0) {
	    if(DEBUG_MODE){
	      fprintf(stderr, 
		  "Parse RTP audio: final:%d, l:%d\n",
			option->generic.final, option->generic.length);
	    }
	    ERROR = TRUE;
	    goto after_parsing;
	  }

	  switch (option->generic.type) {
	    case RTP_SSRC:
	      identifier = ntohs(option->ssrc.id);
	      break;
	    case RTP_APP:
	      /* No option defined */
	      break;
	    default:
	      if(DEBUG_MODE) {
		fprintf(stderr, "Parse RTP audio: bad RTP type %d\n", 
			option->generic.type);
	      }
	      ERROR = TRUE;
	      goto after_parsing;
	      return;
	  }
	} while (!option->generic.final);
    }

  after_parsing:
  if(ERROR) {
    if(DEBUG_MODE){
#ifdef CRYPTO
      fprintf(stderr, "current crypto method: %d\n", current_method);
#endif
      if((host=gethostbyaddr((char *)(&(from.sin_addr)), 
			     sizeof(struct in_addr), 
			     from.sin_family)) ==NULL){
	strcpy(name, inet_ntoa(from.sin_addr));
      }else{
	strcpy(name, host->h_name);
      }
      fprintf(stderr, " from %s\n", name);
    }
    return ;
  }

  h->seq=htons(h->seq);
  h->ts_sec=htons(h->ts_sec);
  h->ts_usec=htons(h->ts_usec);
  
  seq = h->seq;
  *sin_addr=(u_long)from.sin_addr.s_addr;
  *identifi=identifier; 
  DECODING_REQUIRED = ShallIDecode(*sin_addr, identifier, &i_station);

  sendtime.tv_sec=(0x0000ffff & (u_short)ntohs(h->ts_sec));
  sendtime.tv_usec=(0x00ffff0 & ((u_short)ntohs(h->ts_usec) << 4));
  
  gettimeofday(&realtime, (struct timezone *)NULL);
  sendtime.tv_sec=(long)((realtime.tv_sec & 0xffff0000)
			 | (0x0000ffff & sendtime.tv_sec));
  
  
/* Test if we must decode and stock the audio packet */
  if (DECODING_REQUIRED && !AUDIO_DROPPED &&
      i_station!=-1 &&
      (table[i_station].p_paquet==0 || table[i_station].p_paquet->sendtime <
       (sendtime.tv_sec+sendtime.tv_usec/1000000.0))) {

      longueur=DecodeAudio((unsigned char *)pt,lr-((char *)pt-(char *)h),
			   h->format,outbuf);
      paquet.data = (u_char *)malloc ((u_int16) (4*((longueur / 4) + 4)));
/* if we can do the allocation we include the packet in the list */
      if (paquet.data==0) {
	  fprintf(stderr,"Impossible to allocate memory paquet lost\n");
      } else {
	  memcpy((char *)paquet.data, (char *)outbuf, (int)longueur);
	  paquet.beginoftalkspurt = h->s;
	  paquet.validaudio=1;
	  paquet.reserved=0;
	  paquet.actualplay=0;
	  paquet.lastarrived=seq;
	  paquet.sendtime=sendtime.tv_sec+1/1000000.0*sendtime.tv_usec;
	  paquet.arrivaltime=realtime.tv_sec+realtime.tv_usec/1000000.0;
	  paquet.playout=0;
	  paquet.longueur=longueur;
	  table[i_station].p_paquet=(paquet_t *)
	    Insere(table[i_station].p_paquet, &paquet);
	  ORDO_REQUESTED = TRUE;
#ifdef DEBUG
	  TRACE.emetteur=paquet.sendtime;
	  TRACE.recepteur=paquet.arrivaltime;
	  TRACE.begin_of_t=paquet.beginoftalkspurt;
	  TRACE.longueur=paquet.longueur;
	  TRACE.numero=paquet.lastarrived;
	  TRACE.adress=*sin_addr;
	  fwrite((char *)(&TRACE),sizeof(struct trace),1,file_trace);
#endif	      
	  
      }
    }
} 






/* This procedure take data and insert them at the good place */
/* for each station */
/* asock_r is the audio socket recieve number */
void recoit(sin_addr,identifier)
     u_long * sin_addr;
     u_short * identifier;
{
  fd_set fdset;
  static struct timeval timeout={0,0};

  FD_ZERO(&fdset);
  FD_SET(asock_r, &fdset);
  if (select(asock_r+1, &fdset, (fd_set *) NULL, (fd_set *) NULL, &timeout)
      < 0)
    fprintf(stderr,"Pbeme select\n");
  if (FD_ISSET(asock_r, &fdset))
      reception(sin_addr,identifier);
}


static int ordonnance(BUFFER_AUDIO, OLDLOCALPLAY, GLOBALPLAY)
     u_char * BUFFER_AUDIO;
     int  * OLDLOCALPLAY;
     int * GLOBALPLAY;
{
  
  register int i;
  int BESTYOUNG, NOSTATION, NOBODY;
  double temps;
  int ajoue,reste;
  BOOLEAN PLACE=FALSE;
  static BOOLEAN FIRST=TRUE;
  static u_char vide;
  int LOCALPLAY;
  struct timeval realtime;

  if (FIRST) {
      vide = l14_to_pcmu(0);
      memset((char *)BUFFER_AUDIO, (int)vide,BUFFER_AUDIO_LENGTH);
      FIRST=FALSE;
  }
  
  BESTYOUNG=-1;
  NOSTATION=-1;

  for (i=0;i<nb_stations;i++) {
      while (
	     table[i].p_paquet!=0 &&
	     table[i].p_paquet->suivant!=0 &&
	     ((table[i].p_paquet->suivant->lastarrived ==
	      (u_int16)(table[i].p_paquet->lastarrived +1 % 0xFFFF))
	      || (table[i].p_paquet->validaudio==0)))
	{
	  temps=algo(&(table[i].parameter),
		     table[i].p_paquet->suivant->arrivaltime,
		     table[i].p_paquet->suivant->sendtime,
		     (int)table[i].p_paquet->beginoftalkspurt);

	  if (table[i].p_paquet->beginoftalkspurt) {
	      gettimeofday(&realtime, (struct timezone *)NULL);
	      ajoue =(int) ((temps-
			     (realtime.tv_sec+realtime.tv_usec/1000000.0))*
			    8000.0);
	      if (ajoue<0)
/* if the delay is too small we keep the packet but play immediatly */
		table[i].p_paquet->suivant->actualplay=
		  *OLDLOCALPLAY % BUFFER_AUDIO_LENGTH;
	      else {
		if (ajoue+(int)table[i].p_paquet->suivant->longueur>=
		    BUFFER_AUDIO_LENGTH)
		  ajoue=BUFFER_AUDIO_LENGTH-
		    table[i].p_paquet->suivant->longueur-1;
	      }
	      if (AUDIO_SET && ajoue>0) {
		if (ajoue<soundplayqueue())
		  table[i].p_paquet->suivant->actualplay=*OLDLOCALPLAY %
		    BUFFER_AUDIO_LENGTH;
		else
		  table[i].p_paquet->suivant->actualplay=
		    ((ajoue-soundplayqueue())+*OLDLOCALPLAY) %
		    BUFFER_AUDIO_LENGTH;
	      }else
		table[i].p_paquet->suivant->actualplay=
		  (*OLDLOCALPLAY) % BUFFER_AUDIO_LENGTH;
	      LOCALPLAY=table[i].p_paquet->suivant->actualplay;
	      PLACE=TRUE;
	    }
	  else {
	      if (((table[i].p_paquet->actualplay-*OLDLOCALPLAY+
		    BUFFER_AUDIO_LENGTH)%BUFFER_AUDIO_LENGTH)+
		  table[i].p_paquet->suivant->longueur*
		  (table[i].p_paquet->suivant->lastarrived-
		   table[i].p_paquet->lastarrived)
		  < BUFFER_AUDIO_LENGTH)
		{
		  LOCALPLAY=(table[i].p_paquet->actualplay
			     +table[i].p_paquet->suivant->longueur*
			     (table[i].p_paquet->suivant->lastarrived-
			      table[i].p_paquet->lastarrived-1))%
			      BUFFER_AUDIO_LENGTH;
		  PLACE=TRUE;
		} else {
		    if (table[i].p_paquet->validaudio==FALSE) {
		      LOCALPLAY=*OLDLOCALPLAY;
		      PLACE=TRUE;
		    } else {
			PLACE=FALSE;
			break;
		    }
		}
	    }
/* if the variable PLACE is TRUE we must keep the packet an then place it */
	  if (PLACE) {
              /* we see if we are at the end of the buffer */
	      reste=BUFFER_AUDIO_LENGTH-
		(table[i].p_paquet->suivant->longueur+LOCALPLAY);
	      if (reste < 0) {
		  mix2_pcmu((size_t)(BUFFER_AUDIO_LENGTH-LOCALPLAY),
			    &(BUFFER_AUDIO[LOCALPLAY]),
			    table[i].p_paquet->suivant->data);
		  mix2_pcmu((size_t)((table[i].p_paquet->suivant->longueur+
				   LOCALPLAY)-BUFFER_AUDIO_LENGTH),
			    BUFFER_AUDIO,
			 &(table[i].p_paquet->suivant->data[BUFFER_AUDIO_LENGTH
							   -LOCALPLAY]));
		}/* end of if(reste<0) */
	      else
		mix2_pcmu((size_t)table[i].p_paquet->suivant->longueur,
			  &(BUFFER_AUDIO[LOCALPLAY]),
			  table[i].p_paquet->suivant->data);
	      LOCALPLAY = (LOCALPLAY+
              (int)table[i].p_paquet->suivant->longueur) % BUFFER_AUDIO_LENGTH;
	      table[i].p_paquet->suivant->actualplay=LOCALPLAY;
	      /* the next packet was played so we remove the packet */
	      table[i].p_paquet=(paquet_t *)RemoveEnTete(table[i].p_paquet);
	    } /* end of if (place) */
	}/* end of while */
    }/* end of for */
  

/* we detect if station is active*/
  for (i=0;i<nb_stations;i++) {
      if (table[i].audio_decoding && table[i].p_paquet!=0 &&
	  table[i].p_paquet->validaudio) {
	  if (BESTYOUNG<0) {
	      BESTYOUNG=table[i].p_paquet->actualplay;
	      NOSTATION=i;
	  }/* it is the first time */
	  else
	    if ((table[i].p_paquet->actualplay-
		 *OLDLOCALPLAY+BUFFER_AUDIO_LENGTH)%
		BUFFER_AUDIO_LENGTH <=
		(BESTYOUNG-*OLDLOCALPLAY+BUFFER_AUDIO_LENGTH)%
		BUFFER_AUDIO_LENGTH)
	      {
		BESTYOUNG=table[i].p_paquet->actualplay;
		NOSTATION=i;
	      }
	}
    }
  
  if (BESTYOUNG<0)
    *GLOBALPLAY=*OLDLOCALPLAY;
  else {
      *GLOBALPLAY=BESTYOUNG;
      if (table[NOSTATION].p_paquet->beginoftalkspurt)
	table[NOSTATION].p_paquet->validaudio=0;
    }

  NOBODY=1;
  for (i=0;i<nb_stations;i++) {
      if (table[i].p_paquet!=0 && table[i].p_paquet->validaudio) {
	  NOBODY=0;
	  break;
      }
  }
  /* si il n'y aplus de donnees valide alors on remplit de blanc*/
  /* if no data are alive where fill with blank the next begin of talkspurt*/
  if (NOBODY) {
      if (soundplayqueue() <= 0)
	DEBUT=TRUE;
  }
  for (i=0;i<nb_stations;i++) {
      if (table[i].p_paquet!=0 && table[i].p_paquet->beginoftalkspurt)
	table[i].p_paquet->validaudio=0;
  }  
  return NOSTATION;
}/* end of procedure ordonnancement*/



static void Timeout(NOSTATION, OLDLOCALPLAY, GLOBALPLAY)
     int NOSTATION, OLDLOCALPLAY, GLOBALPLAY;
{

  int i;
  paquet_t paquet;
  

/* a timeout is possible only if the audio is available*/  
  if(AUDIO_SET) {
      if ((i=soundplayqueue()) <= LONGUEUR_DEMANDE*2/5
	  && i>=0 && NOSTATION>=0 && GLOBALPLAY==OLDLOCALPLAY) {
	  /* we try to have the lesser number of blank */

	  for (i=0;i<nb_stations;i++) {
	      if (table[i].p_paquet!=0 && table[i].p_paquet->actualplay==
		  OLDLOCALPLAY) {
		  if (table[i].p_paquet->suivant &&
		      ((u_int16)(table[i].p_paquet->lastarrived+2 % 0xFFFF)==
		       table[i].p_paquet->suivant->lastarrived))
		    /*we insert a false packet  */
		    {
		      table[i].p_paquet->beginoftalkspurt=0;
		      paquet.validaudio=1;
		      paquet.beginoftalkspurt=0;
		      paquet.reserved=0;
		      paquet.actualplay=0;
		      paquet.longueur=table[i].p_paquet->longueur;
		      paquet.lastarrived=
			(u_int16)(table[i].p_paquet->lastarrived+1 % 0xFFFF);
		      paquet.sendtime=table[i].p_paquet->sendtime+
			(double)paquet.longueur/8000.0;
		      paquet.arrivaltime=table[i].p_paquet->arrivaltime+
			(double)paquet.longueur/8000.0;
		      paquet.playout=0;
		      paquet.data = (u_char *) malloc ((unsigned)
						       (4*((paquet.longueur
							    / 4) + 4)));
		      if (paquet.data==0) {
			  fprintf(stderr,
				  "Impossible to allocate memory paquet lost\n"
				  );
		      }
		      else {
			  memcpy((char *)paquet.data,
				 (char *)table[i].p_paquet->data,
				 (int)(paquet.longueur));
			  table[i].p_paquet=(paquet_t *)
			    Insere(table[i].p_paquet, &paquet);
		      }
		    }
		  else
		    table[i].p_paquet->beginoftalkspurt=1;
		  table[i].p_paquet->validaudio=0;
		}/*end of if*/
	    }/*end of for*/
	}/* end of if soundplayqueue()<LONGUEUR_DEMANDE*2/5... */
    }/* end of if (AUDIO_SET)*/
  else {
      for (i=0;i<nb_stations;i++) {
	  if (table[i].p_paquet) {
	      table[i].p_paquet->validaudio=0;
	      table[i].p_paquet->beginoftalkspurt=1;
	  }
      }
      /* we say that all packet stocked are loss*/
      DEBUT=TRUE;
  }
}/* end of procedure Timeout*/


      
/* This procedure splits a buffer in small parts */
static void soundplayh(len, buf)
     int len;
     unsigned char *buf;

{
  static len_trans=160;
  register int i;
  
  for (i=0;i<(len-len_trans);i+=len_trans)
      soundplay(len_trans,(unsigned char *)&(buf[i]));
  soundplay(len-i,(unsigned char *)&(buf[i]));

}/*end of procedure sounplayh()*/



/* This procedure plays the BUFFER_AUDIO */

static void joue(BUFFER_AUDIO,oldlocalplay, globalplay)
     u_char * BUFFER_AUDIO;
     int * oldlocalplay;
     int * globalplay;
{
  int lastpos;
  int OLDLOCALPLAY=*oldlocalplay;
  int GLOBALPLAY=*globalplay;
  static BOOLEAN FIRST=TRUE;
  static unsigned char vide;
  static unsigned char BLANC[2000];

  if (FIRST) {
      vide = l14_to_pcmu(0);
      memset((char *)BLANC, (int)vide,2000);
      FIRST=FALSE;
    }

  if (GLOBALPLAY!=OLDLOCALPLAY) {
      if (DEBUT && AUDIO_SET) {
	  DEBUT=FALSE;
	  lastpos=LONGUEUR_DEMANDE -
		   (GLOBALPLAY-OLDLOCALPLAY+
		    BUFFER_AUDIO_LENGTH)
		   %BUFFER_AUDIO_LENGTH;
#if defined(SPARCSTATION) || defined(SGISTATION)
	  if (soundplayqueue()< lastpos)
	      soundplayh(lastpos,BLANC);
#endif	  
	}/*end of if (DEBUT) */
      lastpos=GLOBALPLAY;
      if (lastpos-OLDLOCALPLAY < 0) {
	  if(AUDIO_SET) {
	      soundplayh(BUFFER_AUDIO_LENGTH-OLDLOCALPLAY,
			 &(BUFFER_AUDIO[OLDLOCALPLAY]));
	      soundplayh(lastpos,BUFFER_AUDIO);
	  }
	  memset((char *)&(BUFFER_AUDIO[OLDLOCALPLAY]), (int)vide,
		 BUFFER_AUDIO_LENGTH-OLDLOCALPLAY);
	  memset((char *)BUFFER_AUDIO, (int)vide, lastpos);
      } else {
	  if(AUDIO_SET)
	      soundplayh(lastpos-OLDLOCALPLAY,&(BUFFER_AUDIO[OLDLOCALPLAY]));
	  memset((char *)&(BUFFER_AUDIO[OLDLOCALPLAY]), (int)vide,
		 lastpos-OLDLOCALPLAY);
      }
      if (DEBUT && AUDIO_SET)
	soundplayh(1,&vide);
      OLDLOCALPLAY=GLOBALPLAY;
    }/* fin de if (GLOBALPLAY!= .... */
  else {
      if (DEBUT && AUDIO_SET)
	soundplayh(1,&vide);
  }
  *oldlocalplay=OLDLOCALPLAY;
} /* end of procedure joue()*/


      



/*----------------- procedures for mixing --------------------*/

/* Part of this procedure comes from Nevot, Henning Sculzrinne */

static short  pcmu_l14_table[256];
static u_char l14_pcmu_table[8192];   /* only positive side */

/********************************
***                           ***
***   p c m u _ t o _ l 1 4   ***
***                           ***
*********************************
Convert 8-bit mu-law to 14-bit signed linear (-8031 .. 8031).
*/
static short pcmu_to_l14(pcm)
register int pcm;	/* u-Law PCM value to convert to linear */
{
  register int sign;    /* save sign of PCM value */
  register int segment; /* segment of piecewise linear curve (0..7) */
  register int step;	/* step along segment (0..15) */
  register short lin;	/* return linear value */

  pcm = ~pcm & 0xFF;                  /* undo 1's complement of PCM */
  sign = pcm & 0x80;                  /* extract sign bit */
  pcm = pcm & 0x7F;                   /* retain only magnitude */
  segment = pcm >> 4;                 /* separate into segment */
  step = pcm & 0xF;                   /*  and step within segment */
  lin = ((2*step+33)<<segment)-33;    /* calculate linear value by formula */
  if (sign) lin = -lin;               /* apply the sign */
  return lin;
} /* pcmu_to_l14 */


/********************************
***                           ***
***   l 1 4 _ t o _ p c m u   ***
***                           ***
*********************************
Convert 14-bit linear code (+/- 8158) to 8-bit mu-law.
*/
static u_char l14_to_pcmu(lin)
register short lin;		     /* Linear value to convert to u-Law PCM */
{
    register int decision;		/* Threshold for segment, step */
    register int segment;		/* Segment of piecewise linear curve */
    register int step;			/* Step along segment */
    register int sign = 0;		/* Assume positive sign */

    if (lin < 0) {			/* If negative linear value, */
	lin = -lin;			/*  make positive and */
	sign = 0x80;			/*  get sign to apply later */
    }
    if (lin > 8158) lin = 8158;		/* Limit to maximum linear value */
    lin += 33;				/* Shift up against 8192 */
    for (segment=7, decision=1<<(7+5); segment>=0; segment--, decision>>=1) 
	if (decision < lin) break;	/* Find first seg the value is above */
    lin -= decision;			/* Get offset from segment base */
    step = lin / (decision>>4);		/* Calculate step along segment */
    return(~(sign + (segment<<4) + step) & 0xFF);   /* Assemble PCM value */
} /* l14_to_pcmu */



/******************************************
***                                     ***
***   p c m u _ l i n e a r _ i n i t   ***
***                                     ***
*******************************************
Initialize fast lookup tables for 8, 14, and 16 bit linear values.
*/
static void pcmu_linear_init()
{
  int u,l;

  for (u = 0; u < 256;  u++) {
    pcmu_l14_table[u] = pcmu_to_l14(u);
  }
  for (l = 0; l < 8192; l++) l14_pcmu_table[l] = l14_to_pcmu(l);
} /* pcmu_linear_init */


static u_char sum_table[256*256];		/* table of sums */

/**************************
***                     ***
***   m i x _ i n i t   ***
***                     ***
***************************
Initialize mixing table.
*/
static void mix_init()
{
  register int pcm0,pcm1;			/* Input PCM values */

  /* Fill in the sum table with the sum for each possible combination
     of PCM inputs. Can't use macro since values of > 8192 are
     possible. */
      
  for (pcm0 = 0; pcm0 < 256; pcm0++) {
    for (pcm1 = 0; pcm1 < 256; pcm1++) {
      sum_table[pcm0*256 + pcm1] = 
        l14_to_pcmu(pcmu_to_l14(pcm0) + pcmu_to_l14(pcm1));
    }
  }
}

/*****************************
***                        ***
***   m i x 2 _ p c m u    ***
***                        ***
******************************
Mix two PCM u-law streams.
*/
static void mix2_pcmu(len,v0,v1)
size_t len;   /* number of bytes */
u_char *v0;   /* destination */ 
u_char *v1;   /* stream to be mixed in */
{
  register int i;                             /* Loop index */
   
  for (i = len; i > 0; i--, v0++)
    *v0 = sum_table[(((*v0) << 8) | (*v1++))];
} /* mix2_pcmu */




/*--------------------------- IVS LISTE  UTILITIES -------------------------*/

/* This procedure Insere inserte a packet in a ordered linked liste */
/* it classify the paquet with the sendtime */
static paquet_t * Insere(ptr_tete, a_inserer)
     paquet_t * ptr_tete;
     paquet_t * a_inserer;
{
  paquet_t * ptr, * ptr_aux;
  ptr=ptr_tete;
  /*if it is the List is blank the head pointer is null */
  if (ptr==0)
    {
      ptr=(paquet_t *)malloc((sizeof(paquet_t)/4+1)*4);
      if (ptr==0)
	{
	  fprintf(stderr,"Impossible to allocate memory\n");
	  return ptr;
	}
/*      *ptr=*a_inserer;*/

      ptr->validaudio=a_inserer->validaudio;
      ptr->beginoftalkspurt=a_inserer->beginoftalkspurt;
      ptr->reserved=a_inserer->reserved;
      ptr->reserved2=a_inserer->reserved2;
      ptr->actualplay=a_inserer->actualplay;
      ptr->lastarrived=a_inserer->lastarrived;
      ptr->sendtime=a_inserer->sendtime;
      ptr->arrivaltime=a_inserer->arrivaltime;
      ptr->playout=a_inserer->playout;
      ptr->longueur=a_inserer->longueur;
      ptr->data=a_inserer->data;
      ptr->suivant=0;
      return ptr;
    }
  /* if it is the same paquet we kill it */
  if (ptr->sendtime==a_inserer->sendtime)
    {
      free((char *)a_inserer->data);
      return ptr_tete;
    }
  /* if the packet is the first of the list */
  if ((ptr->sendtime > a_inserer->sendtime))
    {
      ptr_aux=(paquet_t *)malloc((sizeof(paquet_t)/4+1)*4);
      if (ptr_aux==0)
	{
	  fprintf(stderr,"Impossible to allocate memory\n");
	  return ptr;
	}      
/*      *ptr_aux=*a_inserer;*/
      ptr_aux->validaudio=a_inserer->validaudio;
      ptr_aux->beginoftalkspurt=a_inserer->beginoftalkspurt;
      ptr_aux->reserved=a_inserer->reserved;
      ptr_aux->reserved2=a_inserer->reserved2;
      ptr_aux->actualplay=a_inserer->actualplay;
      ptr_aux->lastarrived=a_inserer->lastarrived;
      ptr_aux->sendtime=a_inserer->sendtime;
      ptr_aux->arrivaltime=a_inserer->arrivaltime;
      ptr_aux->playout=a_inserer->playout;
      ptr_aux->longueur=a_inserer->longueur;
      ptr_aux->data=a_inserer->data;
      ptr_aux->suivant=ptr;
      return ptr_aux;
    }
  /* else we're looking for is place */
  while (ptr->suivant!=0)
    {
      if (ptr->suivant->sendtime==a_inserer->sendtime)
	{
	  free((char *)a_inserer->data);
	  return ptr_tete;
	}
      if ((ptr->suivant->sendtime > a_inserer->sendtime))
	{
	  ptr_aux=(paquet_t *)malloc((sizeof(paquet_t)/4+1)*4);
	  if (ptr_aux==0)
	    {
	      fprintf(stderr,"Impossible to allocate memory\n");
	      return ptr_tete;
	    }
/*	  *ptr_aux=*a_inserer;*/
	  ptr_aux->validaudio=a_inserer->validaudio;
	  ptr_aux->beginoftalkspurt=a_inserer->beginoftalkspurt;
	  ptr_aux->reserved=a_inserer->reserved;
	  ptr_aux->reserved2=a_inserer->reserved2;
	  ptr_aux->actualplay=a_inserer->actualplay;
	  ptr_aux->lastarrived=a_inserer->lastarrived;
	  ptr_aux->sendtime=a_inserer->sendtime;
	  ptr_aux->arrivaltime=a_inserer->arrivaltime;
	  ptr_aux->playout=a_inserer->playout;
	  ptr_aux->longueur=a_inserer->longueur;
	  ptr_aux->data=a_inserer->data;
	  ptr_aux->suivant=ptr->suivant;
	  ptr->suivant=ptr_aux;
	  return ptr_tete;
	}
      ptr=ptr->suivant;
    }
/* The packet is the last */
  ptr_aux=(paquet_t *)malloc((sizeof(paquet_t)/4+1)*4);
  if (ptr_aux==0)
    {
      fprintf(stderr,"Impossible to allocate memory\n");
      return ptr_tete;
    }
/*  *ptr_aux=*a_inserer;*/
  ptr_aux->validaudio=a_inserer->validaudio;
  ptr_aux->beginoftalkspurt=a_inserer->beginoftalkspurt;
  ptr_aux->reserved=a_inserer->reserved;
  ptr_aux->reserved2=a_inserer->reserved2;
  ptr_aux->actualplay=a_inserer->actualplay;
  ptr_aux->lastarrived=a_inserer->lastarrived;
  ptr_aux->sendtime=a_inserer->sendtime;
  ptr_aux->arrivaltime=a_inserer->arrivaltime;
  ptr_aux->playout=a_inserer->playout;
  ptr_aux->longueur=a_inserer->longueur;
  ptr_aux->data=a_inserer->data;
  ptr_aux->suivant=0;
  ptr->suivant=ptr_aux;
  return ptr_tete;
}      

/* This function remove the first packet of the list */
/* it return the new Head pointer */
static paquet_t * RemoveEnTete(ptr_tete)
     paquet_t * ptr_tete;
{
  return (paquet_t *)Remove(ptr_tete,ptr_tete);
}/*end of RemoveEnTete*/


/* this function remove the packet pointed by ptr_paquet in the list */
/* where ptr_tete is the head pointer */
/* it return the new Head pointer */
static paquet_t * Remove(ptr_tete,ptr_paquet)
     paquet_t * ptr_tete;
     paquet_t * ptr_paquet;
{
  paquet_t * ptr;
  ptr=ptr_tete;

  /* if the list is empty we remove nothing */
  if (ptr==0)
    return 0;
  /* if it is the head */
  if (ptr==ptr_paquet)
    {
      ptr=ptr_tete->suivant;
      free((char *)ptr_tete->data);
      free((char *)ptr_tete);
      return ptr;
    }
  /* we research the packet and destroy him */
  while (ptr->suivant!=0)
    {
      if (ptr->suivant==ptr_paquet)
	{
	  ptr->suivant=ptr_paquet->suivant;
	  free((char *)ptr_paquet->data);
	  free((char *)ptr_paquet);
	  return ptr_tete;
	}
      ptr=ptr->suivant;
    }
  return ptr_tete;
}


/* this function initialize the list*/
static paquet_t * InitList()
{
  return (paquet_t *)0;
}


/* This function destroy all the packet in the list */
static paquet_t * PurgeList(ptr_tete)
     paquet_t * ptr_tete;
{
  paquet_t * aux=ptr_tete;

  while ((aux=RemoveEnTete(aux))!=0);

  return 0; 
}/*end of function PurgeList*/

#else
BOOLEAN IsAudioEncoding(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
{
  return FALSE;
}


BOOLEAN IsAudioDecoding(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;     /* source identifier, set to 0 if no bridge */
			
{
  return FALSE;
}


BOOLEAN IsAudioIgnored(address,ident)
     u_long  address;         /* packet has been received from this address  */
     u_short ident;              /* source identifier, set to 0 if no bridge */
{
  return TRUE;
}


#endif /* ! NO_AUDIO */
